import { computed, ref, watch, toRef } from 'vue'
import { TreeMate, SubtreeNotLoadedError } from 'treemate'
import { useIsMounted, useMergedState } from 'vooks'
import { useFormItem } from '../../_mixins'
import { call, useAdjustedTo } from '../../_utils'

export function useCascader (props) {
  const uncontrolledValueRef = ref(props.defaultValue)
  const controlledValueRef = toRef(props, 'value')
  const mergedValueRef = useMergedState(
    controlledValueRef,
    uncontrolledValueRef
  )
  const formItem = useFormItem(props)
  const cascaderMenuRef = ref(null)
  const selectMenuRef = ref(null)
  const triggerRef = ref(null)
  const keyboardKeyRef = ref(null)
  const hoverKeyRef = ref(null)
  const loadingKeySetRef = ref(new Set())
  const addLoadingKey = (key) => {
    loadingKeySetRef.value.add(key)
  }
  const deleteLoadingKey = (key) => {
    loadingKeySetRef.value.delete(key)
  }
  const treeMateRef = computed(() => {
    return TreeMate(props.options, {
      getKey (node) {
        return node.value
      }
    })
  })
  const mergedKeysRef = computed(() => {
    const { cascade, multiple } = props
    if (multiple) {
      return treeMateRef.value.getCheckedKeys(mergedValueRef.value, {
        cascade
      })
    } else {
      return {
        checkedKeys: [],
        indeterminateKeys: []
      }
    }
  })
  const checkedKeysRef = computed(() => mergedKeysRef.value.checkedKeys)
  const indeterminateKeysRef = computed(
    () => mergedKeysRef.value.indeterminateKeys
  )
  const menuModelRef = computed(() => {
    const { treeNodePath, treeNode } = treeMateRef.value.getPath(
      hoverKeyRef.value
    )
    let ret
    if (treeNode === null) {
      ret = [treeMateRef.value.treeNodes]
    } else {
      ret = treeNodePath.map((treeNode) => treeNode.siblings)
      if (!treeNode.isLeaf && !loadingKeySetRef.value.has(treeNode.key)) {
        ret.push(treeNode.children)
      }
    }
    return ret
  })
  const hoverKeyPathRef = computed(() => {
    const { keyPath } = treeMateRef.value.getPath(hoverKeyRef.value)
    return keyPath
  })
  watch(props.options, (value, oldValue) => {
    if (!(value === oldValue)) {
      hoverKeyRef.value = null
      keyboardKeyRef.value = null
    }
  })
  function doUpdateValue (value) {
    const { 'onUpdate:value': onUpdateValue, onChange } = props
    const { nTriggerFormInput, nTriggerFormChange } = formItem
    if (onUpdateValue) call(onUpdateValue, value)
    if (onChange) call(onChange, value)
    uncontrolledValueRef.value = value
    nTriggerFormInput()
    nTriggerFormChange()
  }
  function updateKeyboardKey (key) {
    keyboardKeyRef.value = key
  }
  function updateHoverKey (key) {
    hoverKeyRef.value = key
  }
  function doCheck (key) {
    const { cascade, multiple, leafOnly } = props
    if (multiple) {
      try {
        const { checkedKeys } = treeMateRef.value.check(
          key,
          mergedKeysRef.value.checkedKeys,
          {
            cascade,
            leafOnly
          }
        )
        doUpdateValue(checkedKeys)
      } catch (err) {
        if (err instanceof SubtreeNotLoadedError) {
          if (cascaderMenuRef.value) {
            const node = treeMateRef.value.getNode(key)
            if (node !== null) {
              cascaderMenuRef.value.showErrorMessage(node.rawNode.label)
            }
          }
        } else {
          throw err
        }
      }
    } else {
      if (leafOnly) {
        const node = treeMateRef.value.getNode(key)
        if (node !== null && node.isLeaf) {
          doUpdateValue(key)
        } else {
          return false
        }
      } else {
        doUpdateValue(key)
      }
    }
    return true
  }
  function doUncheck (key) {
    const { cascade, multiple, leafOnly } = props
    if (multiple) {
      const { checkedKeys } = treeMateRef.value.uncheck(
        key,
        mergedKeysRef.value.checkedKeys,
        {
          cascade,
          leafOnly
        }
      )
      doUpdateValue(checkedKeys)
    }
  }
  const selectedOptionsRef = computed(() => {
    if (props.multiple) {
      const { showPath, separator } = props
      const { value } = mergedValueRef
      if (Array.isArray(value)) {
        const { getNode } = treeMateRef.value
        return value.map((key) => {
          const node = getNode(key)
          if (node === null) {
            return {
              label: key,
              value: key
            }
          } else {
            return {
              label: showPath
                ? getPathLabel(node, separator)
                : node.rawNode.label,
              value: node.rawNode.value
            }
          }
        })
      } else {
        return []
      }
    } else return []
  })
  const selectedOptionRef = computed(() => {
    const { multiple, showPath, separator } = props
    if (!multiple) {
      const { value } = mergedValueRef
      const { getNode } = treeMateRef.value
      if (value === null) {
        return null
      }
      const node = getNode(value)
      if (node === null) {
        return {
          label: value,
          value
        }
      } else {
        return {
          label: showPath ? getPathLabel(node, separator) : node.rawNode.label,
          value: node.rawNode.value
        }
      }
    } else return null
  })
  const uncontrolledShowRef = ref(false)
  const controlledShowRef = toRef(props, 'show')
  return {
    mergedValue: mergedValueRef,
    uncontrolledShow: uncontrolledShowRef,
    mergedShow: useMergedState(controlledShowRef, uncontrolledShowRef),
    treeMate: treeMateRef,
    keyboardKey: keyboardKeyRef,
    hoverKey: hoverKeyRef,
    menuModel: menuModelRef,
    checkedKeys: checkedKeysRef,
    indeterminateKeys: indeterminateKeysRef,
    selectedOptions: selectedOptionsRef,
    selectedOption: selectedOptionRef,
    hoverKeyPath: hoverKeyPathRef,
    loadingKeySet: loadingKeySetRef,
    isMounted: useIsMounted(),
    cascaderMenuRef,
    selectMenuRef,
    triggerRef,
    selectMenuFollowerRef: ref(null),
    cascaderMenuFollowerRef: ref(null),
    pattern: ref(''),
    adjustedTo: useAdjustedTo(props),
    ...formItem,
    doUpdateValue,
    updateKeyboardKey,
    updateHoverKey,
    doCheck,
    doUncheck,
    addLoadingKey,
    deleteLoadingKey
  }
}

function getPathLabel (node, separator) {
  const path = []
  while (node) {
    path.push(node.rawNode.label)
    node = node.parent
  }
  return path.reverse().join(separator)
}
